﻿using ApiLog.Act.Models.Log;
using ApiLog.Act.Models.Mongo;
using MongoDB.Driver;
using PushPlatform.Common.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace ApiLog.Act.DAL.Mongo
{
    public class APIWebLogsMongRepository
    {
        private MongoLogRepository mongoRep = new MongoLogRepository();
        //private MongoRepository mongoRep = new MongoRepository();

        /// <summary>
        /// 集合名称
        /// </summary>
        private static readonly string _CollectionName = "APIWebLogs";

        #region 新增请求日志

        /// <summary>
        /// 新增请求日志
        /// </summary>
        /// <param name="param"></param>
        /// <returns></returns>
        public async Task<AddRequestLogModel> AddRequestLogAsync(AddRequestLogParam param)
        {
            APIWebLogs entity = new APIWebLogs();
            ObjectUtil.CopyPropertys(param, entity);
            entity.ID = param.TraceID;
            entity.RequestTime = DateTime.Now;
            mongoRep.Add<APIWebLogs>(entity);
            return new AddRequestLogModel() { RequestTime = entity.RequestTime, TraceID = param.TraceID };
        }

        #endregion

        #region 新增响应日志

        /// <summary>
        /// 新增响应日志
        /// </summary>
        /// <param name="param"></param>
        /// <returns></returns>
        public void AddResponseLog(AddResponseLogParam param)
        {
            var builder = Builders<APIWebLogs>.Filter;
            var filter = builder.Eq(nameof(APIWebLogs.ID), param.ParentTraceID);
            var update = Builders<APIWebLogs>.Update
                .Set(nameof(APIWebLogs.ResponseBody), param.ResponseBody)
                .Set(nameof(APIWebLogs.ResponseTime), DateTime.Now)
                .Set(nameof(APIWebLogs.TimeCost), param.TimeCost);

            if (param.IsError)
            {
                update = update.Set(nameof(APIWebLogs.IsError), param.IsError)
                    .Set(nameof(APIWebLogs.ErrorBody), param.ErrorBody)
                    .Set(nameof(APIWebLogs.ErrorBody), param.ErrorBody);
            }

            mongoRep.Update(_CollectionName.ToLower(),filter, update);
        }



        /// <summary>
        /// 查询日志信息
        /// </summary>
        /// <param name="aPIWebLogs"></param>
        /// <returns></returns>
        public async Task<SearchLogsPaggingAsyncOutput> SearchLogsPaggingAsync(SearchAPIWebLogs aPIWebLogs)
        {
            SearchLogsPaggingAsyncOutput searchLogsPaggingAsyncOutput = new SearchLogsPaggingAsyncOutput();
            Expression<Func<APIWebLogs, bool>> expre = x => x.RequestTime > aPIWebLogs.BeginDate && x.RequestTime < aPIWebLogs.EndDate&&x.IsError==aPIWebLogs.IsError;
            Expression<Func<APIWebLogs, object>> sort = x => x.RequestTime;
            if (aPIWebLogs.APIName != null && aPIWebLogs.APIName != "") {
                expre = x => x.APIName == aPIWebLogs.APIName && x.RequestTime > aPIWebLogs.BeginDate && x.RequestTime < aPIWebLogs.EndDate && x.IsError == aPIWebLogs.IsError;
            }
            var result=await mongoRep.PagingQueryAndOrderWithoutBaseEntity<APIWebLogs>(expre, sort, aPIWebLogs.pageIndex, aPIWebLogs.PageSize,true);
            var count= mongoRep.GetCountWithoutBaseEntity<APIWebLogs>(expre);
            searchLogsPaggingAsyncOutput.aPIWebLogs = result;
            searchLogsPaggingAsyncOutput.Count = (int)count;
            return searchLogsPaggingAsyncOutput;
        }

        #endregion
    }
}
